@@ -27,7 +27,10 @@
 #include <vector>
 
 using namespace std;
-using namespace boost;
+using boost::shared_ptr;
+using boost::get;
+using boost::tuple;
+using boost::lexical_cast;
 
 // FIXME: the bogus-cert family of functions is ridiculous
 // and needs to be replaced, or at least factored.
@@ -86,7 +89,7 @@ erase_bogus_certs(vector< manifest<cert> > & certs,
   vector< manifest<cert> > tmp_certs;
 
   // sorry, this is a crazy data structure
-  typedef tuple< hexenc<id>, cert_name, base64<cert_value> > trust_key;
+  typedef boost::tuple< hexenc<id>, cert_name, base64<cert_value> > trust_key;
   typedef map< trust_key, pair< shared_ptr< set<rsa_keypair_id> >, it > > trust_map;
   trust_map trust;
 
@@ -296,7 +299,7 @@ void
 cert_signable_text(cert const & t,
                        string & out)
 {
-  out = (F("[%s@%s:%s]") % t.name % t.ident % remove_ws(t.value())).str();
+  out = (boost::format("[%s@%s:%s]") % t.name % t.ident % remove_ws(t.value())).str();
   L(F("cert: signable text %s\n") % out);
 }
 
@@ -366,7 +369,7 @@ load_priv_key(app_state & app,
             {
               // We really don't want the database key and the rcfile key
               // to differ.
-              N(remove_ws(dbkey()) == remove_ws(luakey()),
+              N(keys_match(id, dbkey, id, luakey),
                   F("mismatch between private key '%s' in database"
                     " and get_priv_key hook") % id);
             }
@@ -391,7 +394,7 @@ calculate_cert(app_state & app, cert & t)
 
   load_priv_key(app, t.key, priv);
 
-  make_signature(app.lua, t.key, priv, signed_text, t.sig);
+  make_signature(app, t.key, priv, signed_text, t.sig);
 }
 
 cert_status 
@@ -419,7 +422,7 @@ check_cert(app_state & app, cert const & t)
 
   string signed_text;
   cert_signable_text(t, signed_text);
-  if (check_signature(app.lua, t.key, pub, signed_text, t.sig))
+  if (check_signature(app, t.key, pub, signed_text, t.sig))
     return cert_ok;
   else
     return cert_bad;
@@ -488,6 +491,7 @@ guess_branch(revision_id const & ident,
           "please provide a branch name") % ident);
       
       decode_base64(certs[0].inner().value, branchname);
+      app.set_branch(branchname());
     }
 }
 
@@ -565,11 +569,11 @@ string const comment_cert_name = "comment";
 string const testresult_cert_name = "testresult";
 
 
-static void 
-cert_revision_date(revision_id const & m, 
-                   boost::posix_time::ptime t,
-                   app_state & app,
-                   packet_consumer & pc)
+void 
+cert_revision_date_time(revision_id const & m, 
+                        boost::posix_time::ptime t,
+                        app_state & app,
+                        packet_consumer & pc)
 {
   string val = boost::posix_time::to_iso_extended_string(t);
   put_simple_revision_cert(m, date_cert_name, val, app, pc);
@@ -584,7 +588,7 @@ cert_revision_date_time(revision_id const & m,
   // make sure you do all your CVS conversions by 2038!
   boost::posix_time::ptime tmp(boost::gregorian::date(1970,1,1), 
                                boost::posix_time::seconds(static_cast<long>(t)));
-  cert_revision_date(m, tmp, app, pc);
+  cert_revision_date_time(m, tmp, app, pc);
 }
 
 void 
@@ -592,7 +596,7 @@ cert_revision_date_now(revision_id const & m,
                        app_state & app,
                        packet_consumer & pc)
 {
-  cert_revision_date(m, boost::posix_time::second_clock::universal_time(), app, pc);
+  cert_revision_date_time(m, boost::posix_time::second_clock::universal_time(), app, pc);
 }
 
 void 
@@ -619,8 +623,7 @@ cert_revision_author_default(revision_id const & m,
         % app.branch_name);
       author = key();
     }
-  put_simple_revision_cert(m, author_cert_name, 
-                           author, app, pc);
+  cert_revision_author(m, author, app, pc);
 }
 
 void 
