@@ -43,7 +43,6 @@ my $man = 0;
 my $user_database = undef;
 my $root = undef;
 my @user_branches = ();
-my @user_not_branches = ();
 my $update = -1;
 my $mail = -1;
 my $attachments = 1;
@@ -56,14 +55,12 @@ my $since = undef;
 my $workdir = undef;
 my $quiet = 0;
 my $debug = 0;
-my $monotone = "mtn";
 
 GetOptions('help|?' => \$help,
 	   'man' => \$man,
 	   'db=s' => \$user_database,
 	   'root=s' => \$root,
 	   'branch=s' => \@user_branches,
-	   'not-branch=s' => \@user_not_branches,
 	   'update!' => \$update,
 	   'mail!' => \$mail,
 	   'attachments!' => \$attachments,
@@ -75,8 +72,7 @@ GetOptions('help|?' => \$help,
 	   'since=s' => \$since,
 	   'workdir=s' => \$workdir,
 	   'quiet' => \$quiet,
-	   'debug' => \$debug,
-	   'monotone=s' => \$monotone) or pod2usage(2);
+	   'debug' => \$debug) or pod2usage(2);
 
 $SIG{HUP} = \&my_exit;
 $SIG{KILL} = \&my_exit;
@@ -107,7 +103,7 @@ if ($mail || $debug) {
 	pod2usage(2);
     }
     if (!defined $difflogs_to && !defined $nodifflogs_to) {
-	my_errlog("You need to specify a To address with --difflogs-to or --nodifflogs-to");
+	my_errlog("You need to specify a To address with --diffslogs-to or --nodiffslogs-to");
 	pod2usage(2);
     }
 }
@@ -116,17 +112,17 @@ if ($mail || $debug) {
 # Make sure we have a database, and that the file spec is absolute.
 #
 
-# If no database is given, check the monotone options file (_MTN/options).
+# If no database is given, check the monotone options file (MT/options).
 # Do NOT use the branch option from there.
 if (!defined $user_database) {
     $root = rel2abs($root) if defined $root;
     $root = rootdir() unless defined $root;
 
     my $curdir = rel2abs(curdir());
-    while(! -f catfile($curdir, "_MTN", "options") && $curdir ne $root) {
+    while(! -f catfile($curdir, "MT", "options") && $curdir ne $root) {
 	$curdir = updir($curdir);
     }
-    my $options = catfile($curdir, "_MTN", "options");
+    my $options = catfile($curdir, "MT", "options");
 
     my_debug("found options file $options");
 
@@ -169,14 +165,6 @@ if ($#user_branches >= 0) {
 			     s/\*/.\*/g;
 			     $_ } @user_branches).')$';
 }
-my $not_branches_re = "^\$";
-if ($#user_not_branches >= 0) {
-    $not_branches_re=
-	'^('.join('|', map { s/([^a-zA-Z0-9\[\]\*\?_])/\\$1/g;
-			     s/\?/./g;
-			     s/\*/.\*/g;
-			     $_ } @user_not_branches).')$';
-}
 my_debug("using the regular expression /$branches_re/ to select branches");
 
 my @files_to_clean_up = ();
@@ -193,9 +181,8 @@ my_debug("changed current directory to $workdir");
 my %branches =
     map { $_ => 1 }
 	grep (/$branches_re/,
-	      grep (!/$not_branches_re/,
-		    map { chomp; $_ }
-			my_backtick("$monotone$database list branches")));
+	      map { chomp; $_ }
+		  my_backtick("monotone$database list branches"));
 my_debug("collected the following branches:\n",
 	 map { "  $_\n" } keys %branches);
 
@@ -206,7 +193,7 @@ my_log("finding all current leaves.");
 # Format: revision => { branch1 => 1, branch2 => 1, ... }
 my %current_leaves = ();
 foreach my $branch (keys %branches) {
-    foreach my $revision (my_backtick("$monotone$database automate heads $branch")) {
+    foreach my $revision (my_backtick("monotone$database automate heads $branch")) {
 	chomp $revision;
 	$current_leaves{$revision} = {} if !defined $current_leaves{$revision};
 	$current_leaves{$revision}->{$branch} = 1;
@@ -220,7 +207,7 @@ my_debug("found ", list_size(keys %current_leaves)," current leaves");
 #
 my_log("finding all old leaves.");
 my %old_leaves = ();
-foreach my $notify_entry (my_backtick("$monotone$database list vars notify")) {
+foreach my $notify_entry (my_backtick("monotone$database list vars notify")) {
     chomp $notify_entry;
     if ($notify_entry =~ /^notify:\s([0-9a-fA-F]{40})\@([^\s]+)\s1$/) {
 	# Found the new format that keeps track of which branches each
@@ -257,7 +244,7 @@ if ($mail || $debug) {
     my_log("collecting all revision IDs between current and old leaves.");
     my %revisions =
 	map { chomp; $_ => 1 }
-	    map { my_backtick("$monotone$database automate ancestry_difference $_ -@ $old_leaves_file") }
+	    map { my_backtick("monotone$database automate ancestry_difference $_ -@ $old_leaves_file") }
 		keys %current_leaves;
     push @files_to_clean_up, "$old_leaves_file";
     my @revisions_keys = keys %revisions;
@@ -279,7 +266,7 @@ if ($mail || $debug) {
     foreach my $revision (keys %revisions) {
 	$revision_data{$revision} =
 	    [ map { chomp; $_ }
-	      my_backtick("$monotone$database log --no-graph --last=1 --from=$revision") ];
+	      my_backtick("monotone$database log --last=1 --revision=$revision") ];
 	my $date = (split(' ', (grep(/^Date:/, @{$revision_data{$revision}}))[0]))[1];
 
 	if (defined $before && $date ge $before) {
@@ -310,11 +297,9 @@ if ($mail || $debug) {
 		my $to = $sendinfo->[2];
 		next if !defined $to;
 
-		# Depending on mtn version, the line will start with
-		# "Ancestor:" or "Parent:"
 		my @ancestors =
 		    map { (split ' ')[1] }
-			grep(/^(Ancestor|Parent):/, @{$revision_data{$revision}});
+			grep(/^Ancestor:/, @{$revision_data{$revision}});
 
 		# If this revision has more than one ancestor, it's the
 		# result of a merge.  If we have already shown the
@@ -446,7 +431,7 @@ if ($mail || $debug) {
 		#
 		if ($mail) {
 		    $msg->send();
-		} elsif ($debug) {
+		} else {
 		    open MESSAGEDBG,">>$debugfile"
 			|| my_error("Couldn't create $debugfile: $!");
 		    print MESSAGEDBG "======================================================================\n";
@@ -485,12 +470,12 @@ my %old_notifications =
 my_log("updating the table of last logged revisions.");
 
 map { my_conditional_system($update,
-			    "$monotone$database set notify $_ 1") }
+			    "monotone$database set notify $_ 1") }
     grep { !defined $old_notifications{$_} && $_ !~ /\@\*$/ }
 	keys %new_notifications;
 map { s/\@\*$//;
       my_conditional_system($update,
-			    "$monotone$database unset notify $_") }
+			    "monotone$database unset notify $_") }
     grep { !defined $new_notifications{$_} }
 	keys %old_notifications;
 
@@ -518,64 +503,42 @@ sub generate_diff
 	print OUTPUT "Diff [$ancestor] -> [$revision]\n";
     }
     if ($ancestor eq "") {
-	if (!$really_show_diffs) {
+	if ($really_show_diffs) {
 	    print OUTPUT "This is the first commit, and there's no easy way to create a diff\n";
 	    print OUTPUT "These are the commands to view the individual files of that commit instead:\n";
 	    print OUTPUT "\n";
 	}
-	my @status = my_backtick("$monotone$db automate get_revision $revision");
-	my $line;
-	$line = shift @status;
-	if ($line =~ /^format_version\s+"([0-9]+)"\s*$/) {
-	    # New versioned format
-	    my $format_version = $1;
-	    if ($format_version == 1) {
-		while($line =~ /^(\s*
-				  |format_version\s+"[0-9]+"
-				  |new_manifest\s+\[[0-9a-f]{40}\]
-				  |old_revision\s+\[[0-9a-f]{40}\]
-				  )\s*$/x) {
-		    $line = shift @status;
-		}
-	    }
-	} else {
-	    while($line =~ /^(\s*
-			      |(new|old)_manifest\s+\[[0-9a-f]{40}\]
-			      |old_revision\s+\[[0-9a-f]{40}\]
-			      )\s*$/x) {
-		$line = shift @status;
-	    }
-	}
+	my @status = my_backtick("monotone$db cat revision $revision");
+	shift @status;	# remove "new_manifest ..."
+	shift @status;	# remove ""
+	shift @status;	# remove "old_revision ..."
+	shift @status;	# remove "old_manifest ..."
 	foreach (@status) {
 	    chomp;
 	    print OUTPUT ($_ eq "" ? "#" : "# $_"), "\n";
 	}
-	my $added_file = "";
+	my $patched_file = "";
 	foreach my $line (@status) {
 	    my $id = undef;
-	    $added_file = $1 if $line =~ /^add_file\s+"(.*)"\s*$/;
-	    $id = $1 if $line =~ /^\s+content\s\[([0-9a-fA-F]{40})\]\s*$/;
-	    # older format had the add_file just name the file, and having
-	    # the content IDs come much later, preceded by a patch line
-	    $added_file = $1 if $line =~ /^patch\s+"(.*)"\s*$/;
-	    $id = $1 if $line =~ /^\s+to\s\[([0-9a-fA-F]{40})\]\s*$/;
+	    $patched_file = $1 if $line =~ /^patch\s+"(.*)"\s*$/;
+	    $id = $1 if $line =~ /^\s+to \[([0-9a-fA-F]{40})\]\s*$/;
 	    if (defined $id) {
 		if ($really_show_diffs) {
-		    my @file = my_backtick("$monotone$db automate get_file $id");
-		    print OUTPUT "--- $added_file\n";
-		    print OUTPUT "+++ $added_file\n";
+		    my @file = my_backtick("monotone$db cat file $id");
+		    print OUTPUT "--- $patched_file\n";
+		    print OUTPUT "+++ $patched_file\n";
 		    print OUTPUT "\@\@ -0,0 +1,",list_size(@file)," \@\@\n";
 		    map { print OUTPUT "+" . $_ } @file;
 		} else {
-		    print OUTPUT "$monotone --db={your.database} automate get_file $id\n";
+		    print OUTPUT "monotone --db={your.database} cat file $id\n";
 		}
 	    }
 	}
     } else {
 	if ($really_show_diffs) {
-	    print OUTPUT my_backtick("$monotone$db diff --revision=$ancestor --revision=$revision");
+	    print OUTPUT my_backtick("monotone$db diff --revision=$ancestor --revision=$revision");
 	} else {
-	    print OUTPUT "mtn --db={your.database} diff --revision=$ancestor --revision=$revision\n";
+	    print OUTPUT "monotone --db={your.database} diff --revision=$ancestor --revision=$revision\n";
 	}
     }
     close OUTPUT;
@@ -595,7 +558,7 @@ sub revision_is_in_branch
     if (!defined $$revision_data{$revision}) {
 	$$revision_data{$revision} =
 	    [ map { chomp; $_ }
-	      my_backtick("$monotone$database log --no-graph --last=1 --from=$revision") ];
+	      my_backtick("monotone$database log --last=1 --revision=$revision") ];
     }
 
     map {
@@ -740,7 +703,7 @@ monotone-notify.pl [--help] [--man]
 [--from=email-sender]
 [--difflogs-to=email-recipient] [--nodifflogs-to=email-recipient]
 [--workdir=path] [--before=yyyy-mm-ddThh:mm:ss] [--since=yyyy-mm-ddThh:mm:ss]
-[--quiet] [--debug] [--monotone=path]
+[--quiet] [--debug]
 
 =head1 DESCRIPTION
 
@@ -764,11 +727,11 @@ Print the manual page and exit.
 =item B<--db>=I<database>
 
 Sets which database to use.  If not given, the database given in
-_MTN/options is used.
+MT/options is used.
 
 =item B<--root>=I<path>
 
-Stop the search for a working copy (containing the F<_MTN> directory) at
+Stop the search for a working copy (containing the F<MT> directory) at
 the specified root directory rather than at the physical root of the
 filesystem.
 
@@ -812,7 +775,7 @@ of the email, separated by lines of dashes.
 
 Do not create difflogs for merges (revisions with more than one
 ancestor), if the ancestors are in at least one of the branches that
-are monitored.  This is the default behavior.
+are monotored.  This is the default behavior.
 
 =item B<--noignore-merges>
 
@@ -871,11 +834,6 @@ small log of it's activities, but with B<--quiet>, it will only output
 error messages.  If B<--debug> was given, B<--quiet> is turned off
 unconditionally.
 
-=item B<--monotone>=I<path>
-
-Gives the name or path to mtn(1) or both.  The default is simply
-F<mtn>.
-
 =back
 
 =head1 NOTES
