@@ -21,17 +21,6 @@
 
 using namespace std;
 
-bool guess_binary(string const & s)
-{
-  // these do not occur in ASCII text files
-  // FIXME: this heuristic is (a) crap and (b) hardcoded. fix both these.
-  if (s.find_first_of("\x00\x01\x02\x03\x04\x05\x06\x0e\x0f"
-                      "\x10\x11\x12\x13\x14\x15\x16\x17\x18"
-                      "\x19\x1a\x1c\x1d\x1e\x1f") != string::npos)
-    return true;
-  return false;
-}
-
 //
 // a 3-way merge works like this:
 //
@@ -96,11 +85,11 @@ struct extent
   edit_t type;
 };
 
-void calculate_extents(vector<long> const & a_b_edits,
-                       vector<long> const & b,
-                       vector<long> & prefix,
+void calculate_extents(vector<long, QA(long)> const & a_b_edits,
+                       vector<long, QA(long)> const & b,
+                       vector<long, QA(long)> & prefix,
                        vector<extent> & extents,
-                       vector<long> & suffix,
+                       vector<long, QA(long)> & suffix,
                        size_t const a_len,
                        interner<long> & intern)
 {
@@ -108,7 +97,7 @@ void calculate_extents(vector<long> const & a_b_edits,
 
   size_t a_pos = 0, b_pos = 0;
 
-  for (vector<long>::const_iterator i = a_b_edits.begin(); 
+  for (vector<long, QA(long)>::const_iterator i = a_b_edits.begin(); 
        i != a_b_edits.end(); ++i)
     {
       // L(F("edit: %d") % *i);
@@ -172,8 +161,8 @@ void calculate_extents(vector<long> const & a_b_edits,
 }
 
 void normalize_extents(vector<extent> & a_b_map,
-                       vector<long> const & a,
-                       vector<long> const & b)
+                       vector<long, QA(long)> const & a,
+                       vector<long, QA(long)> const & b)
 {
   for (size_t i = 0; i < a_b_map.size(); ++i)
     {
@@ -183,39 +172,39 @@ void normalize_extents(vector<extent> & a_b_map,
         while (j > 0
                && (a_b_map.at(j-1).type == preserved)
                && (a_b_map.at(j).type == changed)
-               && (a.at(j) == a.at(j-1))
-               && (b.at(a_b_map.at(j-1).pos) == 
-                   b.at(a_b_map.at(j).pos + a_b_map.at(j).len - 1)))
+               && (a.at(j) == b.at(a_b_map.at(j).pos + a_b_map.at(j).len - 1)))
           {
+            // This is implied by (a_b_map.at(j-1).type == preserved)
+            I(a.at(j-1) == b.at(a_b_map.at(j-1).pos));
 
+            // Coming into loop we have:
+            //                     i
+            //  z   --pres-->  z   0
+            //  o   --pres-->  o   1
+            //  a   --chng-->  a   2   The important thing here is that 'a' in
+            //                 t       the LHS matches with ...
+            //                 u
+            //                 v
+            //                 a       ... the a on the RHS here. Hence we can
+            //  q  --pres-->   q   3   'shift' the entire 'changed' block 
+            //  e  --chng-->   d   4   upwards, leaving a 'preserved' line
+            //  g  --pres-->   g   5   'a'->'a'
             //
-            // Coming into this loop, we've identified this situation:
+            //  Want to end up with:
+            //                     i
+            //  z   --pres-->  z   0
+            //  o   --chng-->  o   1
+            //                 a   
+            //                 t
+            //                 u
+            //                 v
+            //  a  --pres-->   a   2
+            //  q  --pres-->   q   3
+            //  e  --chng-->   d   4
+            //  g  --pres-->   g   5
             //
-            //       A                               B
-            //   --------                    ----------------
-            //   j-1: foo   --preserved-->   mapped[j-1]: foo
-            //     j: foo   --changed---->   mapped[  j]: bar 
-            //                                            foo
-            //
-            // The normalization we want to perform is to move all
-            // "changed" extents to the earliest possible position which
-            // still causes the same B image to be produced.
-            // 
-            //       A                               B
-            //   --------                    ----------------
-            //   j-1: foo   --changed---->   mapped[j-1]: foo
-            //                                            bar
-            //     j: foo   --preserved-->   mapped[  j]: foo
-            //
-
-            // This code is almost untested, because we've only been able to
-            // find one test case that exercises it!  Maybe this text will
-            // make us look bad when/if someone discovers it, but better to
-            // have a bad reputation and good software than vice-versa.
-            W(F("You've found files that trigger a strange edge-case of the merge logic.\n"));
-            W(F("We believe it works, but don't have many tests; so, if you could send us\n"));
-            W(F("a note, and, if possible, the files that triggered it for inclusion in our\n"));
-            W(F("test suite, we'd appreciate it.  Address: " PACKAGE_BUGREPORT ".  Thanks!\n"));
+            // Now all the 'changed' extents are normalised to the
+            // earliest possible position.
 
             L(F("exchanging preserved extent [%d+%d] with changed extent [%d+%d]\n")
               % a_b_map.at(j-1).pos
@@ -270,10 +259,10 @@ void normalize_extents(vector<extent> & a_b_map,
 
 void merge_extents(vector<extent> const & a_b_map,
                    vector<extent> const & a_c_map,
-                   vector<long> const & b,
-                   vector<long> const & c,
+                   vector<long, QA(long)> const & b,
+                   vector<long, QA(long)> const & c,
                    interner<long> const & in,
-                   vector<long> & merged)
+                   vector<long, QA(long)> & merged)
 {
   I(a_b_map.size() == a_c_map.size());
 
@@ -318,7 +307,7 @@ void merge_extents(vector<extent> const & a_b_map,
         }
 
       // mutual or single-edge deletes
-      else if ((i->type == deleted && j->len == deleted)
+      else if ((i->type == deleted && j->type == deleted)
                || (i->type == deleted && j->type == preserved)
                || (i->type == preserved && j->type == deleted))
         { 
@@ -355,13 +344,13 @@ void merge_via_edit_scripts(vector<string> const & ancestor,
                             vector<string> const & right,
                             vector<string> & merged)
 {
-  vector<long> anc_interned;  
-  vector<long> left_interned, right_interned;  
-  vector<long> left_edits, right_edits;  
-  vector<long> left_prefix, right_prefix;  
-  vector<long> left_suffix, right_suffix;  
+  vector<long, QA(long)> anc_interned;  
+  vector<long, QA(long)> left_interned, right_interned;  
+  vector<long, QA(long)> left_edits, right_edits;  
+  vector<long, QA(long)> left_prefix, right_prefix;  
+  vector<long, QA(long)> left_suffix, right_suffix;  
   vector<extent> left_extents, right_extents;
-  vector<long> merged_interned;
+  vector<long, QA(long)> merged_interned;
   interner<long> in;
 
   //   for (int i = 0; i < std::min(std::min(left.size(), right.size()), ancestor.size()); ++i)
@@ -443,7 +432,7 @@ void merge_via_edit_scripts(vector<string> const & ancestor,
   copy(right_suffix.begin(), right_suffix.end(), back_inserter(merged_interned));
 
   merged.reserve(merged_interned.size());
-  for (vector<long>::const_iterator i = merged_interned.begin();
+  for (vector<long, QA(long)>::const_iterator i = merged_interned.begin();
        i != merged_interned.end(); ++i)
     merged.push_back(in.lookup(*i));
 }
@@ -483,7 +472,7 @@ void merge_provider::record_merge(file_id const & left_ident,
   L(F("recording successful merge of %s <-> %s into %s\n")
     % left_ident % right_ident % merged_ident);
 
-  base64< gzip<delta> > merge_delta;
+  delta merge_delta;
   transaction_guard guard(app.db);
 
   diff(left_data.inner(), merged_data.inner(), merge_delta);  
@@ -509,6 +498,18 @@ std::string merge_provider::get_file_encoding(file_path const & path,
     return default_encoding;
 }
 
+bool merge_provider::attribute_manual_merge(file_path const & path,
+                                              manifest_map const & man)
+{
+  std::string mmf;
+  if (get_attribute_from_db(path, manual_merge_attribute, man, mmf, app))
+  {
+    return mmf == std::string("true");
+  }
+  else
+    return false; // default: enable auto merge
+}
+
 bool merge_provider::try_to_merge_files(file_path const & anc_path,
                                         file_path const & left_path,
                                         file_path const & right_path,
@@ -536,68 +537,80 @@ bool merge_provider::try_to_merge_files(file_path const & anc_path,
 
   file_data left_data, right_data, ancestor_data;
   data left_unpacked, ancestor_unpacked, right_unpacked, merged_unpacked;
-  string left_encoding, anc_encoding, right_encoding;
-  vector<string> left_lines, ancestor_lines, right_lines, merged_lines;
 
   this->get_version(left_path, left_id, left_data);
   this->get_version(anc_path, ancestor_id, ancestor_data);
   this->get_version(right_path, right_id, right_data);
 
-  left_encoding = this->get_file_encoding(left_path, left_man);
-  anc_encoding = this->get_file_encoding(anc_path, anc_man);
-  right_encoding = this->get_file_encoding(right_path, right_man);
-    
-  unpack(left_data.inner(), left_unpacked);
-  unpack(ancestor_data.inner(), ancestor_unpacked);
-  unpack(right_data.inner(), right_unpacked);
+  left_unpacked = left_data.inner();
+  ancestor_unpacked = ancestor_data.inner();
+  right_unpacked = right_data.inner();
 
-  split_into_lines(left_unpacked(), left_encoding, left_lines);
-  split_into_lines(ancestor_unpacked(), anc_encoding, ancestor_lines);
-  split_into_lines(right_unpacked(), right_encoding, right_lines);
-    
-  if (merge3(ancestor_lines, 
-             left_lines, 
-             right_lines, 
-             merged_lines))
+  if (!attribute_manual_merge(left_path, left_man) && 
+      !attribute_manual_merge(right_path, right_man))
     {
-      hexenc<id> tmp_id;
-      base64< gzip<data> > packed_merge;
-      string tmp;
-      
-      L(F("internal 3-way merged ok\n"));
-      join_lines(merged_lines, tmp);
-      calculate_ident(data(tmp), tmp_id);
-      file_id merged_fid(tmp_id);
-      pack(data(tmp), packed_merge);
-
-      merged_id = merged_fid;
-      record_merge(left_id, right_id, merged_fid, 
-                   left_data, packed_merge);
-
-      return true;
+      // both files mergeable by monotone internal algorithm, try to merge
+      // note: the ancestor is not considered for manual merging. Forcing the 
+      // user to merge manually just because of an ancestor mistakenly marked
+      // manual seems too harsh
+      string left_encoding, anc_encoding, right_encoding;
+      left_encoding = this->get_file_encoding(left_path, left_man);
+      anc_encoding = this->get_file_encoding(anc_path, anc_man);
+      right_encoding = this->get_file_encoding(right_path, right_man);
+        
+      vector<string> left_lines, ancestor_lines, right_lines, merged_lines;
+      split_into_lines(left_unpacked(), left_encoding, left_lines);
+      split_into_lines(ancestor_unpacked(), anc_encoding, ancestor_lines);
+      split_into_lines(right_unpacked(), right_encoding, right_lines);
+        
+      if (merge3(ancestor_lines, 
+                 left_lines, 
+                 right_lines, 
+                 merged_lines))
+        {
+          hexenc<id> tmp_id;
+          file_data merge_data;
+          string tmp;
+          
+          L(F("internal 3-way merged ok\n"));
+          join_lines(merged_lines, tmp);
+          calculate_ident(data(tmp), tmp_id);
+          file_id merged_fid(tmp_id);
+          merge_data = file_data(tmp);
+    
+          merged_id = merged_fid;
+          record_merge(left_id, right_id, merged_fid, 
+                       left_data, merge_data);
+    
+          return true;
+        }
     }
 
-  P(F("help required for 3-way merge\n"));
-  P(F("[ancestor] %s\n") % anc_path);
-  P(F("[    left] %s\n") % left_path);
-  P(F("[   right] %s\n") % right_path);
-  P(F("[  merged] %s\n") % merged_path);
+  P(F("help required for 3-way merge\n"
+      "[ancestor] %s\n"
+      "[    left] %s\n"
+      "[   right] %s\n"
+      "[  merged] %s\n")
+    % anc_path
+    % left_path
+    % right_path
+    % merged_path);
 
   if (app.lua.hook_merge3(anc_path, left_path, right_path, merged_path,
                           ancestor_unpacked, left_unpacked, 
                           right_unpacked, merged_unpacked))
     {
       hexenc<id> tmp_id;
-      base64< gzip<data> > packed_merge;
+      file_data merge_data;
 
       L(F("lua merge3 hook merged ok\n"));
       calculate_ident(merged_unpacked, tmp_id);
       file_id merged_fid(tmp_id);
-      pack(merged_unpacked, packed_merge);
+      merge_data = file_data(merged_unpacked);
 
       merged_id = merged_fid;
       record_merge(left_id, right_id, merged_fid, 
-                   left_data, packed_merge);
+                   left_data, merge_data);
       return true;
     }
 
@@ -630,28 +643,31 @@ bool merge_provider::try_to_merge_files(file_path const & left_path,
   this->get_version(left_path, left_id, left_data);
   this->get_version(right_path, right_id, right_data);
     
-  unpack(left_data.inner(), left_unpacked);
-  unpack(right_data.inner(), right_unpacked);
+  left_unpacked = left_data.inner();
+  right_unpacked = right_data.inner();
 
-  P(F("help required for 2-way merge\n"));
-  P(F("[    left] %s\n") % left_path);
-  P(F("[   right] %s\n") % right_path);
-  P(F("[  merged] %s\n") % merged_path);
+  P(F("help required for 2-way merge\n"
+      "[    left] %s\n"
+      "[   right] %s\n"
+      "[  merged] %s\n")
+    % left_path
+    % right_path
+    % merged_path);
 
   if (app.lua.hook_merge2(left_path, right_path, merged_path, 
                           left_unpacked, right_unpacked, merged_unpacked))
     {
       hexenc<id> tmp_id;
-      base64< gzip<data> > packed_merge;
+      file_data merge_data;
       
       L(F("lua merge2 hook merged ok\n"));
       calculate_ident(merged_unpacked, tmp_id);
       file_id merged_fid(tmp_id);
-      pack(merged_unpacked, packed_merge);
+      merge_data = file_data(merged_unpacked);
       
       merged_id = merged_fid;
       record_merge(left_id, right_id, merged_fid, 
-                   left_data, packed_merge);
+                   left_data, merge_data);
       return true;
     }
   
@@ -689,10 +705,11 @@ void update_merge_provider::get_version(file_path const & path,
     app.db.get_file_version(ident, dat);
   else
     {
-      base64< gzip<data> > tmp;
+      data tmp;
       file_id fid;
-      N(file_exists (path),
-        F("file %s does not exist in working copy") % path);
+      require_path_is_file(path,
+                           F("file '%s' does not exist in working copy") % path,
+                           F("'%s' in working copy is a directory, not a file") % path);
       read_localized_data(path, tmp, app.lua);
       calculate_ident(tmp, fid);
       N(fid == ident,
@@ -714,7 +731,17 @@ std::string update_merge_provider::get_file_encoding(file_path const & path,
     return default_encoding;
 }
 
-
+bool update_merge_provider::attribute_manual_merge(file_path const & path,
+                                              manifest_map const & man)
+{
+  std::string mmf;
+  if (get_attribute_from_working_copy(path, manual_merge_attribute, mmf))
+    return mmf == std::string("true");
+  else if (get_attribute_from_db(path, manual_merge_attribute, man, mmf, app))
+    return mmf == std::string("true");
+  else
+    return false; // default: enable auto merge
+}
 
 // the remaining part of this file just handles printing out various
 // diff formats for the case where someone wants to *read* a diff
@@ -729,9 +756,9 @@ struct hunk_consumer
   virtual ~hunk_consumer() {}
 };
 
-void walk_hunk_consumer(vector<long> const & lcs,
-                        vector<long> const & lines1,
-                        vector<long> const & lines2,                    
+void walk_hunk_consumer(vector<long, QA(long)> const & lcs,
+                        vector<long, QA(long)> const & lines1,
+                        vector<long, QA(long)> const & lines2,                    
                         hunk_consumer & cons)
 {
 
@@ -749,7 +776,7 @@ void walk_hunk_consumer(vector<long> const & lcs,
   else
     {
       // normal case: files have something in common
-      for (vector<long>::const_iterator i = lcs.begin();
+      for (vector<long, QA(long)>::const_iterator i = lcs.begin();
            i != lcs.end(); ++i, ++a, ++b)
         {         
           if (idx(lines1, a) == *i && idx(lines2, b) == *i)
@@ -832,13 +859,23 @@ void unidiff_hunk_writer::flush_hunk(size_t pos)
         }
       
       // write hunk to stream
-      ost << "@@ -" << a_begin+1;
-      if (a_len > 1)
-        ost << "," << a_len;
+      if (a_len == 0)
+        ost << "@@ -0,0";
+      else
+        {
+          ost << "@@ -" << a_begin+1;
+          if (a_len > 1)
+            ost << "," << a_len;
+        }
       
-      ost << " +" << b_begin+1;
-      if (b_len > 1)
-    ost << "," << b_len;
+      if (b_len == 0)
+        ost << " +0,0";
+      else
+        {
+          ost << " +" << b_begin+1;
+          if (b_len > 1)
+            ost << "," << b_len;
+        }
       ost << " @@" << endl;
       
       copy(hunk.begin(), hunk.end(), ostream_iterator<string>(ost, "\n"));
@@ -1050,14 +1087,16 @@ void cxtdiff_hunk_writer::advance_to(size_t newpos)
 
 void make_diff(string const & filename1,
                string const & filename2,
+               file_id const & id1,
+               file_id const & id2,
                vector<string> const & lines1,
                vector<string> const & lines2,
                ostream & ost,
                diff_type type)
 {
-  vector<long> left_interned;  
-  vector<long> right_interned;  
-  vector<long> lcs;  
+  vector<long, QA(long)> left_interned;  
+  vector<long, QA(long)> right_interned;  
+  vector<long, QA(long)> lcs;  
 
   interner<long> in;
 
@@ -1081,8 +1120,8 @@ void make_diff(string const & filename1,
     {
       case unified_diff:
       {
-        ost << "--- " << filename1 << endl;
-        ost << "+++ " << filename2 << endl;
+        ost << "--- " << filename1 << "\t" << id1 << endl;
+        ost << "+++ " << filename2 << "\t" << id2 << endl;
 
         unidiff_hunk_writer hunks(lines1, lines2, 3, ost);
         walk_hunk_consumer(lcs, left_interned, right_interned, hunks);
@@ -1090,13 +1129,19 @@ void make_diff(string const & filename1,
       }
       case context_diff:
       {
-        ost << "*** " << filename1 << endl;
-        ost << "--- " << filename2 << endl;
+        ost << "*** " << filename1 << "\t" << id1 << endl;
+        ost << "--- " << filename2 << "\t" << id2 << endl;
 
         cxtdiff_hunk_writer hunks(lines1, lines2, 3, ost);
         walk_hunk_consumer(lcs, left_interned, right_interned, hunks);
         break;
       }
+      default:
+      {
+        // should never reach this; the external_diff type is not
+        // handled by this function.
+        I(false);
+      }
     }
 }
 
@@ -1166,8 +1211,8 @@ static void unidiff_append_test()
              + "}\n"
              + "\n");
   
-  string ud(string("--- hello.c\n")
-            + "+++ hello.c\n"
+  string ud(string("--- hello.c\t0123456789abcdef0123456789abcdef01234567\n")
+            + "+++ hello.c\tabcdef0123456789abcdef0123456789abcdef01\n"
             + "@@ -9,3 +9,9 @@\n"
             + " {\n"
             + "         say_hello();\n"
@@ -1183,7 +1228,11 @@ static void unidiff_append_test()
   split_into_lines(src, src_lines);
   split_into_lines(dst, dst_lines);
   stringstream sst;
-  make_diff("hello.c", "hello.c", src_lines, dst_lines, sst, unified_diff);
+  make_diff("hello.c", "hello.c",
+            file_id(id("0123456789abcdef0123456789abcdef01234567")),
+            file_id(id("abcdef0123456789abcdef0123456789abcdef01")),
+            src_lines, dst_lines, sst, unified_diff);
+  cout << sst.str() << std::endl;
   BOOST_CHECK(sst.str() == ud);
 }
 
