@@ -17,9 +17,14 @@
 #include "revision.hh"
 #include "sanity.hh"
 #include "transforms.hh"
+#include "keys.hh"
 
-using namespace boost;
 using namespace std;
+using boost::shared_ptr;
+using boost::lexical_cast;
+using boost::match_default;
+using boost::match_results;
+using boost::regex;
 
 // --- packet db writer --
 //
@@ -116,6 +121,9 @@ public:
     return type < other.type ||
       (type == other.type && ident < other.ident);
   }  
+  // we need to be able to avoid circular dependencies between prerequisite and
+  // delayed_packet shared_ptrs.
+  void cleanup() { delayed.clear(); }
 };
 
 class 
@@ -321,7 +329,7 @@ delayed_revision_data_packet::apply_delayed_packet(packet_db_writer & pw)
 delayed_revision_data_packet::~delayed_revision_data_packet()
 {
   if (!all_prerequisites_satisfied())
-    W(F("discarding revision data packet with unmet dependencies\n"));
+    W(F("discarding revision data packet %s with unmet dependencies\n") % ident);
 }
 
 void 
@@ -334,7 +342,7 @@ delayed_manifest_data_packet::apply_delayed_packet(packet_db_writer & pw)
 delayed_manifest_data_packet::~delayed_manifest_data_packet()
 {
   if (!all_prerequisites_satisfied())
-    W(F("discarding manifest data packet with unmet dependencies\n"));
+    W(F("discarding manifest data packet %s with unmet dependencies\n") % ident);
 }
 
 void 
@@ -366,7 +374,8 @@ delayed_manifest_delta_packet::apply_delayed_packet(packet_db_writer & pw)
 delayed_manifest_delta_packet::~delayed_manifest_delta_packet()
 {
   if (!all_prerequisites_satisfied())
-    W(F("discarding manifest delta packet with unmet dependencies\n"));
+    W(F("discarding manifest delta packet %s -> %s with unmet dependencies\n")
+        % old_id % new_id);
 }
 
 void 
@@ -385,7 +394,8 @@ delayed_file_delta_packet::apply_delayed_packet(packet_db_writer & pw)
 delayed_file_delta_packet::~delayed_file_delta_packet()
 {
   if (!all_prerequisites_satisfied())
-    W(F("discarding file delta packet with unmet dependencies\n"));
+    W(F("discarding file delta packet %s -> %s with unmet dependencies\n")
+        % old_id % new_id);
 }
 
 void 
@@ -428,6 +438,36 @@ delayed_private_key_packet::~delayed_private_key_packet()
   I(all_prerequisites_satisfied());
 }
 
+
+void
+packet_consumer::set_on_revision_written(boost::function1<void,
+                                                        revision_id> const & x)
+{
+  on_revision_written=x;
+}
+
+void
+packet_consumer::set_on_cert_written(boost::function1<void,
+                                                      cert const &> const & x)
+{
+  on_cert_written=x;
+}
+
+void
+packet_consumer::set_on_pubkey_written(boost::function1<void, rsa_keypair_id>
+                                                  const & x)
+{
+  on_pubkey_written=x;
+}
+
+void
+packet_consumer::set_on_privkey_written(boost::function1<void, rsa_keypair_id>
+                                                  const & x)
+{
+  on_privkey_written=x;
+}
+
+
 struct packet_db_writer::impl
 {
   app_state & app;
@@ -459,6 +499,8 @@ struct packet_db_writer::impl
     : app(app), take_keys(take_keys), count(0)
     // cert("cert", 1), manc("manc", 1), manw("manw", 1), filec("filec", 1)
   {}
+
+  ~impl();
 };
 
 packet_db_writer::packet_db_writer(app_state & app, bool take_keys) 
@@ -468,6 +510,26 @@ packet_db_writer::packet_db_writer(app_state & app, bool take_keys)
 packet_db_writer::~packet_db_writer() 
 {}
 
+packet_db_writer::impl::~impl()
+{
+
+  // break any circular dependencies for unsatisfied prerequisites
+  for (map<revision_id, shared_ptr<prerequisite> >::const_iterator i =
+      revision_prereqs.begin(); i != revision_prereqs.end(); i++)
+    {
+      i->second->cleanup();
+    }
+  for (map<manifest_id, shared_ptr<prerequisite> >::const_iterator i =
+      manifest_prereqs.begin(); i != manifest_prereqs.end(); i++)
+    {
+      i->second->cleanup();
+    }
+  for (map<file_id, shared_ptr<prerequisite> >::const_iterator i =
+      file_prereqs.begin(); i != file_prereqs.end(); i++)
+    {
+      i->second->cleanup();
+    }
+}
 
 bool 
 packet_db_writer::impl::revision_exists_in_db(revision_id const & r)
@@ -601,7 +663,7 @@ packet_db_writer::consume_file_delta(file_id const & old_id,
         {
           file_id confirm;
           file_data old_dat;
-          base64< gzip<data> > new_dat;
+          data new_dat;
           pimpl->app.db.get_file_version(old_id, old_dat);
           patch(old_dat.inner(), del.inner(), new_dat);
           calculate_ident(file_data(new_dat), confirm);
@@ -635,7 +697,7 @@ packet_db_writer::consume_file_delta(file_id const & old_id,
 
 void 
 packet_db_writer::consume_file_reverse_delta(file_id const & new_id,
-                                             file_id const & old_id,                                         
+                                             file_id const & old_id,
                                              file_delta const & del)
 {
   transaction_guard guard(pimpl->app.db);
@@ -645,7 +707,7 @@ packet_db_writer::consume_file_reverse_delta(file_id const & new_id,
         {
           file_id confirm;
           file_data new_dat;
-          base64< gzip<data> > old_dat;
+          data old_dat;
           pimpl->app.db.get_file_version(new_id, new_dat);
           patch(new_dat.inner(), del.inner(), old_dat);
           calculate_ident(file_data(old_dat), confirm);
@@ -706,7 +768,7 @@ packet_db_writer::consume_manifest_delta(manifest_id const & old_id,
         {
           manifest_id confirm;
           manifest_data old_dat;
-          base64< gzip<data> > new_dat;
+          data new_dat;
           pimpl->app.db.get_manifest_version(old_id, old_dat);
           patch(old_dat.inner(), del.inner(), new_dat);
           calculate_ident(manifest_data(new_dat), confirm);
@@ -750,7 +812,7 @@ packet_db_writer::consume_manifest_reverse_delta(manifest_id const & new_id,
         {
           manifest_id confirm;
           manifest_data new_dat;
-          base64< gzip<data> > old_dat;
+          data old_dat;
           pimpl->app.db.get_manifest_version(new_id, new_dat);
           patch(new_dat.inner(), del.inner(), old_dat);
           calculate_ident(manifest_data(old_dat), confirm);
@@ -859,6 +921,7 @@ packet_db_writer::consume_revision_data(revision_id const & ident,
       if (dp->all_prerequisites_satisfied())
         {
           pimpl->app.db.put_revision(ident, dat);
+          if(on_revision_written) on_revision_written(ident);
           pimpl->accepted_revision(ident, *this);
         }
     }
@@ -877,6 +940,7 @@ packet_db_writer::consume_revision_cert(revision<cert> const & t)
       if (pimpl->revision_exists_in_db(revision_id(t.inner().ident)))
         {
           pimpl->app.db.put_revision_cert(t);
+          if(on_cert_written) on_cert_written(t.inner());
         }
       else
         {
@@ -911,12 +975,15 @@ packet_db_writer::consume_public_key(rsa_keypair_id const & ident,
       return;
     }
   if (! pimpl->app.db.public_key_exists(ident))
-    pimpl->app.db.put_key(ident, k);
+    {
+      pimpl->app.db.put_key(ident, k);
+      if(on_pubkey_written) on_pubkey_written(ident);
+    }
   else
     {
       base64<rsa_pub_key> tmp;
       pimpl->app.db.get_key(ident, tmp);
-      if (!(tmp() == k()))
+      if (!keys_match(ident, tmp, ident, k))
         W(F("key '%s' is not equal to key '%s' in database\n") % ident % ident);
       L(F("skipping existing public key %s\n") % ident);
     }
@@ -935,7 +1002,10 @@ packet_db_writer::consume_private_key(rsa_keypair_id const & ident,
       return;
     }
   if (! pimpl->app.db.private_key_exists(ident))
-    pimpl->app.db.put_key(ident, k);
+    {
+      pimpl->app.db.put_key(ident, k);
+      if(on_privkey_written) on_privkey_written(ident);
+    }
   else
     L(F("skipping existing private key %s\n") % ident);
   ++(pimpl->count);
@@ -990,6 +1060,38 @@ packet_db_valve::open_valve()
 
 #define DOIT(x) pimpl->do_packet(boost::shared_ptr<delayed_packet>(new x));
 
+void
+packet_db_valve::set_on_revision_written(boost::function1<void,
+                                                        revision_id> const & x)
+{
+  on_revision_written=x;
+  pimpl->writer.set_on_revision_written(x);
+}
+
+void
+packet_db_valve::set_on_cert_written(boost::function1<void,
+                                                      cert const &> const & x)
+{
+  on_cert_written=x;
+  pimpl->writer.set_on_cert_written(x);
+}
+
+void
+packet_db_valve::set_on_pubkey_written(boost::function1<void, rsa_keypair_id>
+                                                  const & x)
+{
+  on_pubkey_written=x;
+  pimpl->writer.set_on_pubkey_written(x);
+}
+
+void
+packet_db_valve::set_on_privkey_written(boost::function1<void, rsa_keypair_id>
+                                                  const & x)
+{
+  on_privkey_written=x;
+  pimpl->writer.set_on_privkey_written(x);
+}
+
 void
 packet_db_valve::consume_file_data(file_id const & ident, 
                                    file_data const & dat)
@@ -1073,8 +1175,10 @@ void
 packet_writer::consume_file_data(file_id const & ident, 
                                  file_data const & dat)
 {
+  base64<gzip<data> > packed;
+  pack(dat.inner(), packed);
   ost << "[fdata " << ident.inner()() << "]" << endl 
-      << trim_ws(dat.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1083,9 +1187,11 @@ packet_writer::consume_file_delta(file_id const & old_id,
                                   file_id const & new_id,
                                   file_delta const & del)
 {
+  base64<gzip<delta> > packed;
+  pack(del.inner(), packed);
   ost << "[fdelta " << old_id.inner()() << endl 
       << "        " << new_id.inner()() << "]" << endl 
-      << trim_ws(del.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1094,9 +1200,11 @@ packet_writer::consume_file_reverse_delta(file_id const & new_id,
                                           file_id const & old_id,
                                           file_delta const & del)
 {
+  base64<gzip<delta> > packed;
+  pack(del.inner(), packed);
   ost << "[frdelta " << new_id.inner()() << endl 
       << "         " << old_id.inner()() << "]" << endl 
-      << trim_ws(del.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1104,8 +1212,10 @@ void
 packet_writer::consume_manifest_data(manifest_id const & ident, 
                                      manifest_data const & dat)
 {
+  base64<gzip<data> > packed;
+  pack(dat.inner(), packed);
   ost << "[mdata " << ident.inner()() << "]" << endl 
-      << trim_ws(dat.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1113,8 +1223,10 @@ void
 packet_writer::consume_revision_data(revision_id const & ident, 
                                      revision_data const & dat)
 {
+  base64<gzip<data> > packed;
+  pack(dat.inner(), packed);
   ost << "[rdata " << ident.inner()() << "]" << endl 
-      << trim_ws(dat.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1123,9 +1235,11 @@ packet_writer::consume_manifest_delta(manifest_id const & old_id,
                                       manifest_id const & new_id,
                                       manifest_delta const & del)
 {
+  base64<gzip<delta> > packed;
+  pack(del.inner(), packed);
   ost << "[mdelta " << old_id.inner()() << endl 
       << "        " << new_id.inner()() << "]" << endl 
-      << trim_ws(del.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1134,9 +1248,11 @@ packet_writer::consume_manifest_reverse_delta(manifest_id const & new_id,
                                               manifest_id const & old_id,
                                               manifest_delta const & del)
 {
+  base64<gzip<delta> > packed;
+  pack(del.inner(), packed);
   ost << "[mrdelta " << new_id.inner()() << endl 
       << "         " << old_id.inner()() << "]" << endl 
-      << trim_ws(del.inner()()) << endl
+      << trim_ws(packed()) << endl
       << "[end]" << endl;
 }
 
@@ -1193,7 +1309,9 @@ feed_packet_consumer
         I(res[3].matched);
         string head(res[1].first, res[1].second);
         string ident(res[2].first, res[2].second);
-        string body(trim_ws(string(res[3].first, res[3].second)));
+        base64<gzip<data> > body_packed(trim_ws(string(res[3].first, res[3].second)));
+        data body;
+        unpack(body_packed, body);
         if (head == "rdata")
           cons.consume_revision_data(revision_id(hexenc<id>(ident)), 
                                      revision_data(body));
@@ -1215,7 +1333,9 @@ feed_packet_consumer
         string head(res[4].first, res[4].second);
         string src_id(res[5].first, res[5].second);
         string dst_id(res[6].first, res[6].second);
-        string body(trim_ws(string(res[7].first, res[7].second)));
+        base64<gzip<delta> > body_packed(trim_ws(string(res[7].first, res[7].second)));
+        delta body;
+        unpack(body_packed, body);
         if (head == "mdelta")
           cons.consume_manifest_delta(manifest_id(hexenc<id>(src_id)), 
                                       manifest_id(hexenc<id>(dst_id)),
@@ -1248,7 +1368,7 @@ feed_packet_consumer
         string certname(res[10].first, res[10].second);
         string key(res[11].first, res[11].second);
         string val(res[12].first, res[12].second);
-        string body(res[13].first, res[13].second);
+        string body(trim_ws(string(res[13].first, res[13].second)));
 
         // canonicalize the base64 encodings to permit searches
         cert t = cert(hexenc<id>(ident),
@@ -1359,20 +1479,16 @@ packet_roundabout_test()
     packet_writer pw(oss);
 
     // an fdata packet
-    base64< gzip<data> > gzdata;
-    pack(data("this is some file data"), gzdata);
-    file_data fdata(gzdata);
+    file_data fdata(data("this is some file data"));
     file_id fid;
     calculate_ident(fdata, fid);
     pw.consume_file_data(fid, fdata);
 
     // an fdelta packet    
-    base64< gzip<data> > gzdata2;
-    pack(data("this is some file data which is not the same as the first one"), gzdata2);
-    file_data fdata2(gzdata2);
+    file_data fdata2(data("this is some file data which is not the same as the first one"));
     file_id fid2;
     calculate_ident(fdata2, fid);
-    base64< gzip<delta> > del;
+    delta del;
     diff(fdata.inner(), fdata2.inner(), del);
     pw.consume_file_delta(fid, fid2, file_delta(del));
 
@@ -1391,7 +1507,7 @@ packet_roundabout_test()
     manifest_map mm;
     manifest_data mdata;
     manifest_id mid;
-    mm.insert(make_pair(file_path("foo/bar.txt"),
+    mm.insert(make_pair(file_path_internal("foo/bar.txt"),
                         file_id(hexenc<id>("cfb81b30ab3133a31b52eb50bd1c86df67eddec4"))));
     write_manifest_map(mm, mdata);
     calculate_ident(mdata, mid);
@@ -1402,13 +1518,13 @@ packet_roundabout_test()
     manifest_data mdata2;
     manifest_id mid2;
     manifest_delta mdelta;
-    mm2.insert(make_pair(file_path("foo/bar.txt"),
+    mm2.insert(make_pair(file_path_internal("foo/bar.txt"),
                          file_id(hexenc<id>("5b20eb5e5bdd9cd674337fc95498f468d80ef7bc"))));
-    mm2.insert(make_pair(file_path("bunk.txt"),
+    mm2.insert(make_pair(file_path_internal("bunk.txt"),
                          file_id(hexenc<id>("54f373ed07b4c5a88eaa93370e1bbac02dc432a8"))));
     write_manifest_map(mm2, mdata2);
     calculate_ident(mdata2, mid2);
-    base64< gzip<delta> > del2;
+    delta del2;
     diff(mdata.inner(), mdata2.inner(), del2);
     pw.consume_manifest_delta(mid, mid2, manifest_delta(del));
     
