@@ -12,6 +12,7 @@
 #include <fstream>
 #include <string>
 #include <vector>
+#include <sstream>
 
 #include <boost/lexical_cast.hpp>
 
@@ -29,7 +30,7 @@ using boost::format;
 sanity global_sanity;
 
 sanity::sanity() : 
-  debug(false), quiet(false), relaxed(false), logbuf(0xffff)
+  debug(false), quiet(false), relaxed(false), logbuf(0xffff), already_dumping(false)
 {
   std::string flavour;
   get_system_flavour(flavour);
@@ -42,19 +43,20 @@ sanity::~sanity()
 void 
 sanity::dump_buffer()
 {
-  if (filename != "")
+  if (!filename.empty())
     {
-      ofstream out(filename.c_str());
+      ofstream out(filename.as_external().c_str());
       if (out)
         {
           copy(logbuf.begin(), logbuf.end(), ostream_iterator<char>(out));
-          ui.inform(string("wrote debugging log to ") + filename + "\n");
+          copy(gasp_dump.begin(), gasp_dump.end(), ostream_iterator<char>(out));
+          ui.inform((F("wrote debugging log to %s") % filename).str());
         }
       else
-        ui.inform("failed to write debugging log to " + filename + "\n");
+        ui.inform((F("failed to write debugging log to %s") % filename).str());
     }
   else
-    ui.inform(string("discarding debug log (maybe you want --debug or --dump?)\n"));
+    ui.inform("discarding debug log (maybe you want --debug or --dump?)");
 }
 
 void 
@@ -74,6 +76,12 @@ sanity::set_debug()
     ui.inform((*i) + "\n");
 }
 
+void
+sanity::set_brief()
+{
+  brief = true;
+}
+
 void 
 sanity::set_quiet()
 {
@@ -87,23 +95,29 @@ sanity::set_relaxed(bool rel)
   relaxed = rel;
 }
 
+string
+sanity::do_format(format const & fmt, char const * file, int line)
+{
+  try
+    {
+      return fmt.str();
+    }
+  catch (std::exception & e)
+    {
+      ui.inform(F("fatal: formatter failed on %s:%d: %s")
+		% file
+		% line
+		% e.what());
+      throw;
+    }
+}
+
+
 void 
 sanity::log(format const & fmt, 
             char const * file, int line)
 {
-  string str;
-  try 
-    {
-      str = fmt.str();
-    }
-  catch (std::exception & e)
-    {
-      ui.inform("fatal: formatter failed on " 
-                + string(file) 
-                + ":" + boost::lexical_cast<string>(line) 
-                + ": " + e.what());
-      throw e;
-    }
+  string str = do_format(fmt, file, line);
   
   if (str.size() > constants::log_line_sz)
     {
@@ -112,6 +126,8 @@ sanity::log(format const & fmt,
         str.at(str.size() - 1) = '\n';
     }
   copy(str.begin(), str.end(), back_inserter(logbuf));
+  if (str[str.size() - 1] != '\n')
+    logbuf.push_back('\n');
   if (debug)
     ui.inform(str);
 }
@@ -120,19 +136,8 @@ void
 sanity::progress(format const & fmt, 
                  char const * file, int line)
 {
-  string str;
-  try 
-    {
-      str = fmt.str();
-    }
-  catch (std::exception & e)
-    {
-      ui.inform("fatal: formatter failed on " 
-                + string(file) 
-                + ":" + boost::lexical_cast<string>(line) 
-                + ": " + e.what());
-      throw e;
-    }
+  string str = do_format(fmt, file, line);
+
   if (str.size() > constants::log_line_sz)
     {
       str.resize(constants::log_line_sz);
@@ -140,6 +145,8 @@ sanity::progress(format const & fmt,
         str.at(str.size() - 1) = '\n';
     }
   copy(str.begin(), str.end(), back_inserter(logbuf));
+  if (str[str.size() - 1] != '\n')
+    logbuf.push_back('\n');
   if (! quiet)
     ui.inform(str);
 }
@@ -148,19 +155,8 @@ void
 sanity::warning(format const & fmt, 
                 char const * file, int line)
 {
-  string str;
-  try 
-    {
-      str = fmt.str();
-    }
-  catch (std::exception & e)
-    {
-      ui.inform("fatal: formatter failed on " 
-                + string(file) 
-                + ":" + boost::lexical_cast<string>(line) 
-                + ": " + e.what());
-      throw e;
-    }
+  string str = do_format(fmt, file, line);
+
   if (str.size() > constants::log_line_sz)
     {
       str.resize(constants::log_line_sz);
@@ -169,6 +165,8 @@ sanity::warning(format const & fmt,
     }
   string str2 = "warning: " + str;
   copy(str2.begin(), str2.end(), back_inserter(logbuf));
+  if (str[str.size() - 1] != '\n')
+    logbuf.push_back('\n');
   if (! quiet)
     ui.warn(str);
 }
@@ -180,7 +178,18 @@ sanity::naughty_failure(string const & expr, format const & explain,
   string message;
   log(format("%s:%d: usage constraint '%s' violated\n") % file % line % expr,
       file.c_str(), line);
-  prefix_lines_with("misuse: ", explain.str(), message);
+  prefix_lines_with(_("misuse: "), explain.str(), message);
+  throw informative_failure(message);
+}
+
+void 
+sanity::error_failure(string const & expr, format const & explain, 
+                        string const & file, int line)
+{
+  string message;
+  log(format("%s:%d: detected error '%s' violated\n") % file % line % expr,
+      file.c_str(), line);
+  prefix_lines_with(_("error: "), explain.str(), message);
   throw informative_failure(message);
 }
 
@@ -192,6 +201,7 @@ sanity::invariant_failure(string const & expr,
     format("%s:%d: invariant '%s' violated\n") 
     % file % line % expr;
   log(fmt, file.c_str(), line);
+  gasp();
   throw logic_error(fmt.str());
 }
 
@@ -206,5 +216,98 @@ sanity::index_failure(string const & vec_expr,
     format("%s:%d: index '%s' = %d overflowed vector '%s' with size %d\n")
     % file % line % idx_expr % idx % vec_expr % sz;
   log(fmt, file.c_str(), line);
+  gasp();
   throw logic_error(fmt.str());
 }
+
+// Last gasp dumps
+
+void
+sanity::gasp()
+{
+  if (already_dumping)
+    {
+      L(F("ignoring request to give last gasp; already in process of dumping\n"));
+      return;
+    }
+  already_dumping = true;
+  L(F("saving current work set: %i items") % musings.size());
+  std::ostringstream out;
+  out << F("Current work set: %i items\n") % musings.size();
+  for (std::vector<MusingI const *>::const_iterator
+         i = musings.begin(); i != musings.end(); ++i)
+    {
+      std::string tmp;
+      try
+        {
+          (*i)->gasp(tmp);
+          out << tmp;
+        }
+      catch (logic_error)
+        {
+          out << tmp;
+          out << "<caught logic_error>\n";
+          L(F("ignoring error trigged by saving work set to debug log"));
+        }
+      catch (informative_failure)
+        {
+          out << tmp;
+          out << "<caught informative_failure>\n";
+          L(F("ignoring error trigged by saving work set to debug log"));
+        }
+    }
+  gasp_dump = out.str();
+  L(F("finished saving work set"));
+  if (debug)
+    {
+      ui.inform("contents of work set:");
+      ui.inform(gasp_dump);
+    }
+  already_dumping = false;
+}
+
+MusingI::MusingI()
+{
+  if (!global_sanity.already_dumping)
+    global_sanity.musings.push_back(this);
+}
+
+MusingI::~MusingI()
+{
+  if (!global_sanity.already_dumping)
+    {
+      I(global_sanity.musings.back() == this);
+      global_sanity.musings.pop_back();
+    }
+}
+
+void
+dump(std::string const & obj, std::string & out)
+{
+  out = obj;
+}
+
+
+void MusingBase::gasp(const std::string & objstr, std::string & out) const
+{
+  out = (boost::format("----- begin '%s' (in %s, at %s:%d)\n"
+		       "%s"
+		       "-----   end '%s' (in %s, at %s:%d)\n")
+	 % name % func % file % line
+	 % objstr
+	 % name % func % file % line
+	 ).str();
+}
+
+
+boost::format F(const char * str)
+{
+  return boost::format(gettext(str), get_user_locale());
+}
+
+
+boost::format FP(const char * str1, const char * strn, unsigned long count)
+{
+  return boost::format(ngettext(str1, strn, count), get_user_locale());
+}
+
