@@ -1,3 +1,4 @@
+// -*- mode: C++; c-file-style: "gnu"; indent-tabs-mode: nil -*-
 // copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
 // all rights reserved.
 // licensed to the public under the terms of the GNU GPL (>= 2)
@@ -11,15 +12,12 @@
 #include <string>
 #include <vector>
 
-#include <boost/filesystem/path.hpp>
 #include <boost/tokenizer.hpp>
+#include <boost/scoped_array.hpp>
 
-#include "cryptopp/filters.h"
-#include "cryptopp/files.h"
-#include "cryptopp/sha.h"
-#include "cryptopp/hex.h"
-#include "cryptopp/base64.h"
-#include "cryptopp/gzip.h"
+#include "botan/botan.h"
+#include "botan/gzip.h"
+#include "botan/sha160.h"
 
 #include "idna/idna.h"
 #include "idna/stringprep.h"
@@ -65,23 +63,61 @@ using namespace std;
 template<typename XFM> string xform(string const & in)
 {
   string out;
-  out.reserve(in.size() * 2);
-  CryptoPP::StringSource 
-    str(in, true, 
-        new XFM(new CryptoPP::StringSink(out)));
+  Botan::Pipe pipe(new XFM());
+  pipe.process_msg(in);
+  out = pipe.read_all_as_string();
   return out;
 }
 
 // specialize it
-template string xform<CryptoPP::Base64Encoder>(string const &);
-template string xform<CryptoPP::Base64Decoder>(string const &);
-template string xform<CryptoPP::HexEncoder>(string const &);
-template string xform<CryptoPP::HexDecoder>(string const &);
-template string xform<CryptoPP::Gzip>(string const &);
-template string xform<CryptoPP::Gunzip>(string const &);
+template string xform<Botan::Base64_Encoder>(string const &);
+template string xform<Botan::Base64_Decoder>(string const &);
+template string xform<Botan::Hex_Encoder>(string const &);
+template string xform<Botan::Hex_Decoder>(string const &);
+template string xform<Botan::Gzip_Compression>(string const &);
+template string xform<Botan::Gzip_Decompression>(string const &);
 
 // for use in hexenc encoding
 
+string encode_hexenc(string const & in)
+{
+  boost::scoped_array<char> buf(new char[in.size() * 2]);
+  static char const *tab = "0123456789abcdef";
+  char *c = buf.get();
+  for (string::const_iterator i = in.begin();
+       i != in.end(); ++i)
+    {
+      *c++ = tab[(*i >> 4) & 0xf];
+      *c++ = tab[*i & 0xf];
+    }
+  return string(buf.get(), in.size() *2);
+}
+
+static inline char decode_hex_char(char c)
+{
+  if (c >= '0' && c <= '9')
+    return c - '0';
+  if (c >= 'a' && c <= 'f')
+    return c - 'a' + 10;
+  I(false);
+}
+
+string decode_hexenc(string const & in)
+{
+  I(in.size() % 2 == 0);
+  boost::scoped_array<char> buf(new char[in.size() / 2]);
+  char *c = buf.get();
+  for (string::const_iterator i = in.begin();
+       i != in.end(); ++i)
+    {
+      char t = decode_hex_char(*i++);
+      t <<= 4;
+      t |= decode_hex_char(*i);
+      *c++ = t;
+    }
+  return string(buf.get(), in.size() / 2);        
+}
+
 struct 
 lowerize
 {
@@ -116,89 +152,79 @@ uppercase(string const & in)
   return n;
 }
 
+template <typename T>
+void pack(T const & in, base64< gzip<T> > & out)
+{
+  string tmp;
+  tmp.reserve(in().size()); // FIXME: do some benchmarking and make this a constant::
+
+  Botan::Pipe pipe(new Botan::Gzip_Compression(), new Botan::Base64_Encoder);
+  pipe.process_msg(in());
+  tmp = pipe.read_all_as_string();
+  out = tmp;
+}
+
+template <typename T>
+void unpack(base64< gzip<T> > const & in, T & out)
+{
+  string tmp;
+  tmp.reserve(in().size()); // FIXME: do some benchmarking and make this a constant::
+
+  Botan::Pipe pipe(new Botan::Base64_Decoder(), new Botan::Gzip_Decompression());
+  pipe.process_msg(in());
+  tmp = pipe.read_all_as_string();
+
+  out = tmp;
+}
+
+// specialise them
+template void pack<data>(data const &, base64< gzip<data> > &);
+template void pack<delta>(delta const &, base64< gzip<delta> > &);
+template void unpack<data>(base64< gzip<data> > const &, data &);
+template void unpack<delta>(base64< gzip<delta> > const &, delta &);
 
 // diffing and patching
 
-
 void 
 diff(data const & olddata,
      data const & newdata,
-     base64< gzip<delta> > & del)
+     delta & del)
 {
   string unpacked;
   compute_delta(olddata(), newdata(), unpacked);
-  pack(delta(unpacked), del);
+  del = delta(unpacked);
 }
 
 void 
 patch(data const & olddata,
-      base64< gzip<delta> > const & del,
+      delta const & del,
       data & newdata)
 {
-  delta unpacked;
-  unpack(del, unpacked);
   string result;
-  apply_delta(olddata(), unpacked(), result);
+  apply_delta(olddata(), del(), result);
   newdata = result;
 }
 
 void 
 diff(manifest_map const & oldman,
      manifest_map const & newman,
-     base64< gzip<delta> > & del)
+     delta & del)
 {
   string xd;
   compute_delta(oldman, newman, xd);
-  pack(delta(xd), del);
+  del = delta(xd);
 }
 
-void 
-diff(base64< gzip<data> > const & olddata,
-     base64< gzip<data> > const & newdata,
-     base64< gzip<delta> > & del)
-{
-  gzip<data> olddata_decoded;
-  gzip<data> newdata_decoded;
-
-  decode_base64(olddata, olddata_decoded);
-  decode_base64(newdata, newdata_decoded);
-
-  data olddata_decompressed;
-  data newdata_decompressed;
-
-  decode_gzip(olddata_decoded, olddata_decompressed);
-  decode_gzip(newdata_decoded, newdata_decompressed);
-
-  diff(olddata_decompressed,
-       newdata_decompressed,
-       del);
-}
-
-void 
-patch(base64< gzip<data> > const & olddata,
-      base64< gzip<delta> > const & del,
-      base64< gzip<data> > & newdata)
-{
-  data olddata_unpacked, newdata_unpacked;
-  unpack(olddata, olddata_unpacked);
-  patch(olddata_unpacked, del, newdata_unpacked);
-  pack(newdata_unpacked, newdata);
-}
-
-
 // identifier (a.k.a. sha1 signature) calculation
 
 void 
 calculate_ident(data const & dat,
                 hexenc<id> & ident)
 {
-  CryptoPP::SHA hash;
-  hash.Update(reinterpret_cast<byte const *>(dat().c_str()), 
-              static_cast<unsigned int>(dat().size()));
-  char digest[CryptoPP::SHA::DIGESTSIZE];
-  hash.Final(reinterpret_cast<byte *>(digest));
-  string out(digest, CryptoPP::SHA::DIGESTSIZE);
-  id ident_decoded(out);
+  Botan::Pipe p(new Botan::Hash_Filter("SHA-1"));
+  p.process_msg(dat());
+
+  id ident_decoded(p.read_all_as_string());
   encode_hexenc(ident_decoded, ident);  
 }
 
@@ -226,7 +252,6 @@ void
 calculate_ident(manifest_map const & m,
                 manifest_id & ident)
 {
-  CryptoPP::SHA hash;
   size_t sz = 0;
   static size_t bufsz = 0;
   static char *buf = NULL;
@@ -235,7 +260,7 @@ calculate_ident(manifest_map const & m,
        i != m.end(); ++i)
     {
       sz += i->second.inner()().size();
-      sz += i->first().size();
+      sz += i->first.as_internal().size();
       sz += 3;      
     }
 
@@ -255,18 +280,15 @@ calculate_ident(manifest_map const & m,
       c += i->second.inner()().size();
       *c++ = ' '; 
       *c++ = ' '; 
-      memcpy(c, i->first().data(), i->first().size());
-      c += i->first().size();
+      memcpy(c, i->first.as_internal().data(), i->first.as_internal().size());
+      c += i->first.as_internal().size();
       *c++ = '\n'; 
     }
   
-  hash.Update(reinterpret_cast<byte const *>(buf), 
-              static_cast<unsigned int>(sz));
+  Botan::Pipe p(new Botan::Hash_Filter("SHA-1"));
+  p.process_msg(reinterpret_cast<Botan::byte const*>(buf), sz);
 
-  char digest[CryptoPP::SHA::DIGESTSIZE];
-  hash.Final(reinterpret_cast<byte *>(digest));
-  string out(digest, CryptoPP::SHA::DIGESTSIZE);
-  id ident_decoded(out);
+  id ident_decoded(p.read_all_as_string());
   hexenc<id> raw_ident;
   encode_hexenc(ident_decoded, raw_ident);  
   ident = manifest_id(raw_ident);    
@@ -286,9 +308,7 @@ void calculate_ident(revision_data const & dat,
                      revision_id & ident)
 {
   hexenc<id> tmp;
-  data unpacked;
-  unpack(dat.inner(), unpacked);
-  calculate_ident(unpacked, tmp);
+  calculate_ident(dat.inner(), tmp);
   ident = tmp;
 }
 
@@ -302,7 +322,7 @@ void calculate_ident(revision_set const & cs,
   ident = tid;
 }
 
-// this might reasonably go in file_io.cc too..
+// this might reasonably go in file_io.cc too...
 void 
 calculate_ident(file_path const & file,
                 hexenc<id> & ident,
@@ -326,13 +346,13 @@ calculate_ident(file_path const & file,
   else
     {
       // no conversions necessary, use streaming form
-      CryptoPP::SHA hash;
-      unsigned int const sz = 2 * CryptoPP::SHA::DIGESTSIZE;
-      char buffer[sz];
-      CryptoPP::FileSource f(file().c_str(), true, new CryptoPP::HashFilter
-                             (hash, new CryptoPP::HexEncoder
-                              (new CryptoPP::ArraySink(reinterpret_cast<byte *>(buffer), sz))));
-      ident = lowercase(string(buffer, sz));
+      // Best to be safe and check it isn't a dir.
+      assert_path_is_file(file);
+      Botan::Pipe p(new Botan::Hash_Filter("SHA-1"), new Botan::Hex_Encoder());
+      Botan::DataSource_Stream infile(file.as_external());
+      p.process_msg(infile);
+
+      ident = lowercase(p.read_all_as_string());
     }
 }
 
@@ -386,8 +406,6 @@ void split_into_lines(std::string const & in,
     {
       out.push_back(in);
     }
-  if (out.size() == 0)
-    out.push_back("");
 }
 
 
@@ -473,8 +491,8 @@ trim_ws(string const & s)
 string 
 canonical_base64(string const & s)
 {
-  return xform<CryptoPP::Base64Encoder>
-    (xform<CryptoPP::Base64Decoder>(s));
+  return xform<Botan::Base64_Encoder>
+    (xform<Botan::Base64_Decoder>(s));
 }
 
 
@@ -504,13 +522,14 @@ charset_convert(string const & src_charset,
       char * converted = stringprep_convert(src.c_str(),
                                             dst_charset.c_str(),
                                             src_charset.c_str());
-      I(converted != NULL);
+      E(converted != NULL,
+        F("failed to convert string from %s to %s: '%s'")
+         % src_charset % dst_charset % src);
       dst = string(converted);
       free(converted);
     }
 }
 
-
 void 
 system_to_utf8(external const & ext, utf8 & utf)
 {
@@ -519,11 +538,89 @@ system_to_utf8(external const & ext, utf8 & utf)
   utf = out;
 }
 
+// hack: this is an unexposed function in libidna
+extern "C" long g_utf8_strlen(const char * p, size_t max);
+
+size_t
+length(utf8 const & utf)
+{
+  return g_utf8_strlen(utf().c_str(), utf().size());
+}
+
+// Lots of gunk to avoid charset conversion as much as possible.  Running
+// iconv over every element of every path in a 30,000 file manifest takes
+// multiple seconds, which then is a minimum bound on pretty much any
+// operation we do...
+static inline bool
+system_charset_is_utf8_impl()
+{
+  std::string lc_encoding = lowercase(system_charset());
+  return (lc_encoding == "utf-8"
+          || lc_encoding == "utf_8"
+          || lc_encoding == "utf8");
+}
+
+static inline bool
+system_charset_is_utf8()
+{
+  static bool it_is = system_charset_is_utf8_impl();
+  return it_is;
+}
+
+static inline bool
+system_charset_is_ascii_extension_impl()
+{
+  if (system_charset_is_utf8())
+    return true;
+  std::string lc_encoding = lowercase(system_charset());
+  // if your character set is identical to ascii in the lower 7 bits, then add
+  // it here for a speed boost.
+  return (lc_encoding.find("ascii") != std::string::npos
+          || lc_encoding.find("8859") != std::string::npos
+          || lc_encoding.find("ansi_x3.4") != std::string::npos
+          // http://www.cs.mcgill.ca/~aelias4/encodings.html -- "EUC (Extended
+          // Unix Code) is a simple and clean encoding, standard on Unix
+          // systems.... It is backwards-compatible with ASCII (i.e. valid
+          // ASCII implies valid EUC)."
+          || lc_encoding.find("euc") != std::string::npos);
+}
+
+static inline bool
+system_charset_is_ascii_extension()
+{
+  static bool it_is = system_charset_is_ascii_extension_impl();
+  return it_is;
+}
+
+inline static bool
+is_all_ascii(string const & utf)
+{
+  // could speed this up by vectorization -- mask against 0x80808080,
+  // process a whole word at at time...
+  for (std::string::const_iterator i = utf.begin(); i != utf.end(); ++i)
+    if (0x80 & *i)
+      return false;
+  return true;
+}
+
+// this function must be fast.  do not make it slow.
+void 
+utf8_to_system(utf8 const & utf, std::string & ext)
+{
+  if (system_charset_is_utf8())
+    ext = utf();
+  else if (system_charset_is_ascii_extension()
+           && is_all_ascii(utf()))
+    ext = utf();
+  else
+    charset_convert("UTF-8", system_charset(), utf(), ext);
+}
+
 void 
 utf8_to_system(utf8 const & utf, external & ext)
 {
   string out;
-  charset_convert("UTF-8", system_charset(), utf(), out);
+  utf8_to_system(utf, out);
   ext = out;
 }
 
@@ -575,7 +672,6 @@ utf8_to_ace(utf8 const & utf, ace & a)
   free(out);
 }
 
-
 void 
 internalize_cert_name(utf8 const & utf, cert_name & c)
 {
@@ -614,9 +710,10 @@ internalize_rsa_keypair_id(utf8 const & utf, rsa_keypair_id & key)
     tokenizer;
   boost::char_separator<char> sep("", ".@", boost::keep_empty_tokens);
   tokenizer tokens(utf(), sep);
+  bool in_domain = false;
   for(tokenizer::iterator i = tokens.begin(); i != tokens.end(); ++i)
     {
-      if (*i == "." || *i == "@")
+      if (!in_domain || *i == "." || *i == "@")
         tmp += *i;
       else
         {
@@ -624,6 +721,8 @@ internalize_rsa_keypair_id(utf8 const & utf, rsa_keypair_id & key)
           utf8_to_ace(*i, a);
           tmp += a();
         }
+      if (*i == "@")
+        in_domain = true;
     }
   key = tmp;
 }
@@ -644,9 +743,10 @@ externalize_rsa_keypair_id(rsa_keypair_id const & key, utf8 & utf)
     tokenizer;
   boost::char_separator<char> sep("", ".@", boost::keep_empty_tokens);
   tokenizer tokens(key(), sep);
+  bool in_domain = false;
   for(tokenizer::iterator i = tokens.begin(); i != tokens.end(); ++i)
     {
-      if (*i == "." || *i == "@")
+      if (!in_domain || *i == "." || *i == "@")
         tmp += *i;
       else
         {
@@ -655,6 +755,8 @@ externalize_rsa_keypair_id(rsa_keypair_id const & key, utf8 & utf)
           ace_to_utf8(a, u);
           tmp += u();
         }
+      if (*i == "@")
+        in_domain = true;
     }
   utf = tmp;
 }
@@ -712,14 +814,15 @@ line_end_convert(string const & linesep, string const & src, string & dst)
   vector<string> tmp;
   split_into_lines(src, tmp);
   join_lines(tmp, dst, linesep_str);
-  if (src.size() >= 1 &&
-      (src[src.size() - 1] == '\r' ||
-       src[src.size() - 1] == '\n'))
+  if (src.size() >= linesep.size() &&
+      (src.compare(src.size() - linesep.size(), linesep.size(), linesep) == 0))
     dst += linesep_str;
 }
 
+
 #ifdef BUILD_UNIT_TESTS
 #include "unit_tests.hh"
+#include <stdlib.h>
 
 static void 
 enc_test()
@@ -740,19 +843,11 @@ rdiff_test()
 {
   data dat1(string("the first day of spring\nmakes me want to sing\n"));
   data dat2(string("the first day of summer\nis a major bummer\n"));
-  data dat3;
-  gzip<data> dat1_gz, dat2_gz, dat3_gz;
-  base64< gzip<data> > dat1_bgz, dat2_bgz, dat3_bgz;
-  encode_gzip(dat1, dat1_gz);
-  encode_gzip(dat2, dat2_gz);
-  encode_base64(dat1_gz, dat1_bgz);
-  encode_base64(dat2_gz, dat2_bgz);
-  base64< gzip<delta> > del_bgz;
-  diff(dat1_bgz, dat2_bgz, del_bgz);
+  delta del;
+  diff(dat1, dat2, del);
   
-  patch(dat1_bgz, del_bgz, dat3_bgz);
-  decode_base64(dat3_bgz, dat3_gz);
-  decode_gzip(dat3_gz, dat3);
+  data dat3;
+  patch(dat1, del, dat3);
   BOOST_CHECK(dat3 == dat2);
 }
 
@@ -992,7 +1087,7 @@ check_idna_encoding()
       ace a = string(idna_vec[i].out);
       ace tace;
       utf8_to_ace(utf, tace);
-      L(F("ACE-encoded %s: '%s'\n") % idna_vec[i].name % tace());
+      L(boost::format("ACE-encoded %s: '%s'\n") % idna_vec[i].name % tace());
       BOOST_CHECK(lowercase(a()) == lowercase(tace()));
       ace_to_utf8(a, tutf);
       BOOST_CHECK(lowercase(utf()) == lowercase(tutf()));
