@@ -5,12 +5,8 @@
 
 #include <string>
 #include <iostream>
-#include <boost/filesystem/path.hpp>
-#include <boost/filesystem/exception.hpp>
-#include <boost/version.hpp>
 
 #include "constants.hh"
-#include "file_io.hh"
 #include "sanity.hh"
 #include "vocab.hh"
 
@@ -18,12 +14,33 @@
 
 using namespace std;
 
+// the verify() stuff gets a little complicated; there doesn't seem to be a
+// really nice way to achieve what we want with c++'s type system.  the
+// problem is this: we want to give verify(file_path) and verify(local_path)
+// access to the internals of file_path and local_path, i.e. make them
+// friends, so they can normalize the file paths they're given.  this means
+// that verify() needs to be declared publically, so that the definition of
+// these classes can refer to them.  it also means that they -- and all other
+// ATOMIC types -- cannot fall back on a templated version of verify if no
+// other version is defined, because, well, the friend thing and the template
+// thing just don't work out, as far as I can tell.  So, every ATOMIC type
+// needs an explicitly defined verify() function, so we have both ATOMIC() and
+// ATOMIC_NOVERIFY() macros, the latter of which defines a type-specific noop
+// verify function.  DECORATE and ENCODING, on the other hand, cannot make use
+// of a trick like these, because they are template types themselves, and we
+// want to be able to define verify(hexenc<id>) without defining
+// verify(hexenc<data>) at the same time, for instance.  Fortunately, these
+// types never need to be friends with their verify functions (yet...), so we
+// _can_ use a templated fallback function.  This templated function is used
+// _only_ by DECORATE and ENCODING; it would be nice to make it take an
+// argument of type T1<T2> to document that, but for some reason that doesn't
+// work either.
 template <typename T>
 static inline void
 verify(T & val)
 {}
 
-static inline void 
+inline void 
 verify(hexenc<id> & val)
 {
   if (val.ok)
@@ -41,7 +58,7 @@ verify(hexenc<id> & val)
   val.ok = true;
 }
 
-static inline void 
+inline void 
 verify(ace & val)
 {
   if (val.ok)
@@ -55,7 +72,7 @@ verify(ace & val)
 }
 
 
-static inline void 
+inline void 
 verify(cert_name & val)
 {
   if (val.ok)
@@ -68,7 +85,7 @@ verify(cert_name & val)
   val.ok = true;
 }
 
-static inline void 
+inline void 
 verify(rsa_keypair_id & val)
 {
   if (val.ok)
@@ -81,75 +98,39 @@ verify(rsa_keypair_id & val)
   val.ok = true;
 }
 
-
-static inline void 
-verify(local_path & val)
+inline void
+verify(netsync_session_key & val)
 {
-
   if (val.ok)
     return;
 
-  using boost::filesystem::path;
-  boost::filesystem::path p;
-  try 
+  if (val().size() == 0)
     {
-      p = mkpath(val());
-#if BOOST_VERSION >= 103100
-      p = p.normalize();
-#endif
-    }
-  catch (std::runtime_error &re)
-    {
-      throw informative_failure(re.what());
-    }
-  catch (fs::filesystem_error &fse)
-    {
-      throw informative_failure(fse.what());
+      val.s.append(constants::netsync_session_key_length_in_bytes, 0);
+      return;
     }
 
-  N(! (p.has_root_path() || p.has_root_name() || p.has_root_directory()),
-    F("prohibited absolute path '%s'") % val);
+  N(val().size() == constants::netsync_session_key_length_in_bytes,
+    F("Invalid key length of %d bytes") % val().length());
 
-  for(path::iterator i = p.begin(); i != p.end(); ++i)
-    {
-      N(!( *i == "" && (! p.empty())),
-        F("empty path component in '%s'") % val);
-
-      N((*i != ".."),
-        F("prohibited path component '%s' in '%s'") % *i % val);
-
-      string::size_type pos = i->find_first_of(constants::illegal_path_bytes);
-      N(pos == string::npos,
-        F("bad character '%d' in path component '%s' of '%s'") 
-        % static_cast<int>(i->at(pos)) % *i % val);
-
-      string s = val();
-      for (string::const_iterator j = s.begin(); j != s.end(); ++j)
-      N(*j != '\0',
-        F("null byte in path component '%s' of '%s'") % *i % val);
-        
-    }
-  
   val.ok = true;
 }
 
-static inline void 
-verify(file_path & val)
+inline void
+verify(netsync_hmac_value & val)
 {
-  static std::set<std::string> known_good;
-
   if (val.ok)
     return;
-  
-  if (known_good.find(val()) == known_good.end())
+
+  if (val().size() == 0)
     {
-      local_path loc(val());
-      verify(loc);
-      N(!book_keeping_file(loc),
-        F("prohibited book-keeping path in '%s'") % val);
-      known_good.insert(val());
+      val.s.append(constants::netsync_hmac_value_length_in_bytes, 0);
+      return;
     }
-  
+
+  N(val().size() == constants::netsync_hmac_value_length_in_bytes,
+    F("Invalid hmac length of %d bytes") % val().length());
+
   val.ok = true;
 }
 
@@ -172,8 +153,12 @@ ty const & ty::operator=(ty const & other)   \
                                              \
 ostream & operator<<(ostream & o,            \
                      ty const & a)           \
-{ return (o << a.s); }
+{ return (o << a.s); }                       \
+                                             \
+void dump(ty const & obj, std::string & out) \
+{ out = obj(); }
 
+#define ATOMIC_NOVERIFY(ty) ATOMIC(ty)
 
 
 
@@ -200,7 +185,11 @@ enc<INNER>::operator=(enc<INNER> const & other)          \
                                                          \
 template <typename INNER>                                \
 ostream & operator<<(ostream & o, enc<INNER> const & e)  \
-{ return (o << e.i); }
+{ return (o << e.i); }                                   \
+                                                         \
+template <typename INNER>                                \
+void dump(enc<INNER> const & obj, std::string & out)     \
+{ out = obj(); }
 
 
 #define DECORATE(dec)                                    \
@@ -222,8 +211,11 @@ dec<INNER>::operator=(dec<INNER> const & other)          \
                                                          \
 template <typename INNER>                                \
 ostream & operator<<(ostream & o, dec<INNER> const & d)  \
-{ return (o << d.i); }
-
+{ return (o << d.i); }                                   \
+                                                         \
+template <typename INNER>                                \
+void dump(dec<INNER> const & obj, std::string & out)     \
+{ dump(obj.inner(), out); }
 
 #define EXTERN 
 
@@ -241,49 +233,10 @@ template class manifest<cert>;
 #ifdef BUILD_UNIT_TESTS
 #include "unit_tests.hh"
 
-static void test_file_path_verification()
-{
-  char const * baddies [] = {"../escape",
-                             "foo/../../escape",
-                             "/rooted",
-                             "foo//nonsense",
-#ifdef _WIN32
-                             "c:\\windows\\rooted",
-                             "c:/windows/rooted",
-                             "c:thing",
-                             "//unc/share",
-                             "//unc",
-#endif
-                             0 };
-  
-  for (char const ** c = baddies; *c; ++c)
-    BOOST_CHECK_THROW(file_path p(*c), informative_failure);      
-  
-  char const * bad = "\t\r\n\v\f\a\b";
-  char badboy[] = "bad";
-  for (char const * c = bad; *c; ++c)
-    {
-      badboy[1] = *c;
-      BOOST_CHECK_THROW(file_path p(badboy), informative_failure);
-    }
-  
-  char const * goodies [] = {"unrooted", 
-                             "unrooted.txt",
-                             "fun_with_underscore.png",
-                             "fun-with-hyphen.tiff", 
-                             "unrooted/../unescaping",
-                             "unrooted/general/path",
-                             "here/..",
-                             0 };
-
-  for (char const ** c = goodies; *c; ++c)
-    BOOST_CHECK_NOT_THROW(file_path p(*c), informative_failure);
-}
-
 void add_vocab_tests(test_suite * suite)
 {
   I(suite);
-  suite->add(BOOST_TEST_CASE(&test_file_path_verification));
+  // None, ATM.
 }
 
 #endif // BUILD_UNIT_TESTS
